--******************************************************************************
--*  Lights Grouper Utility v1.2     for Max 4.2                               *
--*  by Ofer Zelichover (c) 12/2001                                            *
--*  www.tdp.nu/ofer   ;   ofer_z@hotmail.com                                  *
--*                                                                            *
--******************************************************************************
--*  You may use this script freely as you see fit.                            *
--*  You may use parts or the script as a whole in your own scripts.           *
--*  (it would be nice if you give me a credit if you do so ;))                *
--******************************************************************************
--*  This script comes with no waranty!                                        *
--*  Although I tried this script and couldn't find any problems with it, I can*
--*  in no way be held responsible for any kind of loss or damage, whether     *
--*  direct or indirect, due to the use of this script.                        *
--*                                                                            *
--*  ********************************************************************      *
--*  *** IF YOU DON'T LIKE THE ABOVE STATEMENT, DON'T USE THIS SCRIPT ***      *
--*  ********************************************************************      *
--*                                                                            *
--*  **** This script was written for max 4.2 and wasn't tested on other ****  *
--*  **** versions of max.                                               ****  *
--*                                                                            *
--*  If you find any bugs in this script, please let me know.                  *
--******************************************************************************
--* Description                                                                *
--* -------------                                                              *
--* This utility allows you to define groups of lights, for easy management.   *
--* You can then turn all the lights in the group On/Off and Hide/Unhide them  *
--* It is useful for task such as having the same scene with day/night light   *
--* schemes.                                                                   *
--* The new Sets feature allows to create presets of groups and easily switch  *
--* between them. This is useful, for example, if you have a large amount of   *
--* lights in the scene, and you want to create a simple lighting sceme for    *
--* draft renderings. Just create two sets, one for draft and one for          *
--* production, define which lights you want on and which off in each set, and *
--* just switch to the right set before you render.                            *
--*                                                                            *
--******************************************************************************
--* History, Status and Known issues                                           *
--* ----------------------------------                                         *
--* created : 23/8/2001                                                        *
--* update : 25/8/2001 ver0.3 added save and load to max scene                 *
--* update : 27/8/2001 ver0.4 added save and load to ext file                  *
--* update : 01/9/2001 ver0.5 added control over basic light parameters        *
--* update : 5/9/2001 v0.51 added change handlers fixed minor bugs.            *
--* update : 6/9/2001 v0.52 changed the save with max function. now it saves   *
--*          using string streams, each group has one appData entry. appData   *
--*          starts at 33100 now.                                              *
--*          added rolling text in the about rollout.                          *
--*          created message boxes when there are missing lights when loading. *
--* update : 9/9/2001 v0.53 added: when using select all in groups, if there is*
--*          only one group, now updating GUI. Changed the way the groups GUI  *
--*          updates.                                                          *
--* update : 10/9/2001 v1.0 after some testing, it seems to be stable enough   *
--*          to reach version 1.0. as always, it's probably not bug free, but  *
--*          in a working condition.                                           *
--* update : 07/12/2001 v1.1                                                   *
--*    - added GUI features to support the upcoming set feature.               *
--* update : 08/12/2001 v1.11                                                  *
--*    - added supprot for sets. now you have sets, each one holds its own     *
--*      groups.                                                               *
--*    - to support the new sets features the data saving had to be changed.   *
--*      now the data is saved in the globalTracks appData starting at address *
--*      33100 (can be modified with a var below). the data sturcture hold the *
--*      set name in the 00 address, then, each group is saved in a new address*
--*      the next set starts the next round 100 (33200, 33300 etc.). still need*
--*      to implement load\save from\to ext. file. also need to add support for*
--*      loading the old format for backward compatibility.                    *
--* update : 09/12/2001 v1.12                                                  *
--*    - added supprot for loading older data from max scene.                  *
--*    - added new file load/save (new fromat and old one).                    *
--* update : 09/12/2001 v1.13                                                  *
--*    - updated the about rollout. changed the scrolling text to smoother     *
--*      scroll.                                                               *
--* update : 10/12/2001 v1.14                                                  *
--*    - added a progress display when loading light grouper data.             *
--* update : 10/12/2001 v1.15                                                  *
--*    - added an ini file to save the last pos/size and autosave mode.        *
--*                                                                            *
--* Known issues:                                                              *
--*  -if params of lights are changed in the scene it's not reflected in the   *
--*   util.                                                                    *
--*                                                                            *
--*Need to add:                                                                *
--*                                                                            *
--******************************************************************************
--* Isntallation:                                                              *
--* --------------                                                             *
--*    put:                                                                    *
--*        LightsGrouper_v1_2.ms       - anywhere                              *
--*                                                                            *
--*   after you run the script you will find it in the customize user interface*
--*   under "Os Tools" category.                                               *
--*                                                                            *
--******************************************************************************
macroScript LightsGrouper

	category:"Os Tools"
	tooltip:"Lights Grouper Utility v1.2"
	buttontext:"LightGrouper"
(

global lgFloat	-- declare the floater

-- Local Rollouts Declerations
--------------------------------
	local saveLoad
	local lgProgress

-- Local Function Declerations
--------------------------------
	local loadFromExtFile
	local saveToExtFile
	local loadFromMaxScene
	local saveToMaxScene
	local saveIniFile
	local createChangeHandlers

-- Local Variables Declerations
--------------------------------
	local appDataAddr = 33100	-- the data will be save with globalTrack appData starting at this index
	local errNumMissingLights = 0
	

--***********************************************************************
-- lgProgress rollout
--***********************************************************************
rollout lgProgress "Light Grouper"
(
	label lbl "Loading..." align:#left
	progressbar progBar ""
	
	on lgProgress open do (
		progBar.value = 0
	)
) -- end of lgProgress rollout
	
	
--***********************************************************************
-- Main rollout
--***********************************************************************
rollout Main "Light Groups"
(
-- Local Struct Declerations
--=============================================================
	struct groupSet (name, lightGroups)
	struct lightGroup (
		name, 
		lights, 
		enabled=true, 
		hide=false,
		fn copy =(
			local tmpLightGroup = lightGroup name #() enabled hide
			tmpLightGroup.lights = for l in lights collect l
			tmpLightGroup
		)
	)

-- Local Variables Declerations
--=============================================================
	local groupSets = #()
	local lastSetNum = 0
	local lightGroups = #()
	local lastGrpNum = 0
	
-- Local Function Declerations
--=============================================================
	local bitarrayCount, getSelectedItem
	local isSingleItemSelected
	local lightParamBoxEnabled
	local getLightsNames
	local updateAllGroupsMode

-- GUI
--==============================================================
	groupbox grpSetsOutline "" width:188 height:61 pos:[4,5]
	groupbox grpSets "Sets: " width:186 height:60 pos:[5,5]
		dropdownlist mainSetList "" pos:[10,20] items:#() width:175
		button mainAddSet "Add" width:44 height:16 pos:[8,45] toolTip:"Add a new set"
		button mainCopySet "Clone" width:44 height:16 pos:[53,45] toolTip:"Clone the selected set"
		button mainRenameSet "Rename" width:44 height:16 pos:[98,45] toolTip:"Rename the selected"
		button mainDelSet "Remove" width:44 height:16 pos:[143,45] toolTip:"Remove the selected set from the list"
	
	groupbox grpGroupsOutline "" width:188 height:266 pos:[4,70]
	groupbox grpGroups "Groups: " width:186 height:265 pos:[5,70]
		multilistbox mainGroups "" items:#() selection:#() height:5 width:175 pos:[10,85]
		button mainGroupAll "All" pos:[10,160] height:16 width:50 toolTip:"Select all groups in the list."
		button mainGroupNone "None" pos:[72,160] height:16 width:50 toolTip:"Clear selection of the groups in the list."
		button mainGroupInv "Invert" pos:[135,160] height:16 width:50 toolTip:"Invert selection of groups in the list."
		button mainAddGroup "Add Group"  align:#left pos:[10,190] toolTip:"Add a new group"
		button mainDelGroup "Remove Selectd" align:#left pos:[90,190] toolTip:"Remove the selected group(s) from the list"
		edittext mainGrpName "Name: " width:175 pos:[10,218] 
		groupbox grpGroupState "Group(s) State: " width:175 height:40 pos:[10,240]
			checkbutton mainGrpEnable "On" width:60 pos:[25,255] toolTip:"Turn the lights in the selected group(s) On/Off"
			checkbutton mainGrpHide "Hide" width:60 pos:[110,255] toolTip:"Show/Hide the lights in the selected group(s)"
		groupbox grpGroupSelScene "Select in scene: " width:175 height:40 pos:[10,290]
			checkbox mainGrpSelClear "Clear selection" pos:[15,307]
			button mainGrpSelect "Select" pos:[115,305] width:65 toolTip:"Select in the scene the light in the selected group(s)." 
		
	groupbox grpLightsOutline "" width:188 height:416 pos:[193,5] enabled:false
	groupbox grpLights "Lights: " width:186 height:415 pos:[194,5] enabled:false
		multilistbox mainGroupLights "" items:#() selection:#() height:10 width:175 pos:[200,20] enabled:false
		button mainLightAll "All" pos:[200,160] height:16 width:50 enabled:false toolTip:"Select all lights in the list."
		button mainLightNone "None" pos:[262,160] height:16 width:50 enabled:false toolTip:"Clear selection of the lights in the list."
		button mainLightInv "Invert" pos:[325,160] height:16 width:50 enabled:false toolTip:"Invert selection of lights in the list."
		button mainAddLights "Add Lights"  align:#left pos:[200,190] enabled:false toolTip:"Add lights to the group."
		button mainDelLights "Remove Selected" align:#left pos:[275,190] enabled:false toolTip:"Remove selected light(s) from the group."
		groupbox grpLightParam "Light parameters: " width:175 height:200 pos:[200,215]
			button mainLightSelect "S" width:13 height:38 pos:[203,230] toolTip:"Select light in scene"
			checkbox mainLightOn "On" pos:[222,230]
			edittext mainLightName "Name:" pos:[260,230] width:110
			spinner mainLightMult "Multiplier:" range:[-1000.0,1000.0,1.0] type:#float scale:0.1 width:55 pos:[257,250]
			colorpicker mainLightColor "Color:" fieldWidth:20 height:18 pos:[320,249] color:white
			checkbox mainLightShad "Shadow" pos:[205,275]
			dropdownlist mainLightShadType "" items:#("Shadow Maps","Raytraced","VrayShadow","VrayShadowMap") width:100 pos:[270,272]
			label mainLightLabel_1 "MapSize" pos:[205,295]
			spinner mainLightMapSize "" range:[0,100000.0,512.0] type:#integer width:50 pos:[205,310]
			label mainLightLabel_2 "Bias" pos:[270,295]
			spinner mainLightBias "" range:[0.0,1000.0,1.0] type:#float width:40 pos:[270,310]
			label mainLightLabel_3 "Samples" pos:[325,295]
			spinner mainLightSamples "" range:[0.0,1000.0,4.0] type:#float width:40 pos:[325,310]
			-- here is what I added !
			checkbox mainLightTransparentshadows "Transparent Sh." pos:[205,330]
			checkbox mainLightAreashadows "Area Sh." pos:[308,330]
			label mainLightLabel_4 "USize:" pos:[205,350]
			spinner mainLightUSize "" range:[0,100000,10] type:#integer width:50 pos:[235,350]
			label mainLightLabel_5 "VSize:" pos:[205,370]
			spinner mainLightVSize "" range:[0,100000,10] type:#integer width:50 pos:[235,370]
			label mainLightLabel_6 "WSize:" pos:[204,390]
			spinner mainLightWSize "" range:[0,100000,10] type:#integer width:50 pos:[235,390]
			radiobuttons mainLightAreaShadowType "" labels:#("Box", "Sphere") width:10 pos:[305,350] default:2 columns:1
			label mainLightLabel_7 "Subs:" pos:[292,390]
			spinner mainLightSubdivs "" range:[0,100000,8] type:#integer width:50 pos:[320,390]
			-- that's it.

		
	button closeFloater "Close" height:25 width:375 pos:[5,425] toolTip:"Close Lights Grouper."

	
-- Functions
--========================================================
--Filter Functions
----------------------
-- this function save automatically the changes to the max scene.
	fn autoSave = (
		if saveLoad.slAutoSave.state then
			saveToMaxScene()
	)
	
--  These two functions filter the lights in the select object window
	fn isLightUsed obj = (
		local tmpList = list = #()
		for g in lightGroups do (
			tmplist = for l in g.lights collect l
			join list tmpList
		)
		(findItem list obj)!=0
	)
	
	fn pickFilter obj = (isKindOf obj Light and (NOT isLightUsed obj))

-- GUI Functions
---------------------
-- Inverts the selection in a listbox
	fn selectionInvert sel = (
		try (
			for i = 1 to sel.count do sel[i] = Not sel[i]
		) catch()
		sel
	)

-- resets the light parameters display when elements are turned grey/inactive
	fn resetLightParam = (
		mainLightName.text = ""
		mainLightMult.value = 1.0
		mainLightColor.color = white
		mainLightMapSize.value = 512
		mainLightBias.value = 1.0
		mainLightSamples.value = 4.0
		mainLightUsize.value = 10
		mainLightVsize.value = 10
		mainLightWsize.value = 10
		mainLightAreaShadowType.state= 2
		mainLightSubdivs.value = 8
	)

-- updates the data in the light parameters box
	--to see wether the light has vrayshadows or not, i checked if the light had a "smoothshadow" property, which is vrayshadows exclusive
	--to see wether the light has vrayshadowmaps or not, i checked if the light was raytraced (exclusive to vray and mental ray shadows) and if it had a "resolution" property, which is vrayshadows exclusive
	fn updateLightParamGUI lit = (
		mainLightOn.state			= lit.on
		mainLightName.text			= lit.name
		mainLightMult.value 		= lit.multiplier
		mainLightColor.color 		= lit.color
		mainLightShad.state 		= lit.baseobject.castShadows
		mainLightShadType.selection = if lit.raytracedShadows and hasProperty lit "smoothshadows" then 3 else ( if NOT lit.raytracedShadows then 1 else( if lit.raytracedShadows and hasProperty lit "resolution" then 4 else 2))
		mainLightMapSize.value 		= if NOT lit.raytracedShadows then lit.mapSize else if lit.raytracedShadows and hasproperty lit "resolution" then lit.resolution else 512
		mainLightBias.value 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.Bias else(if(lit.raytracedShadows and hasProperty lit "resolution") then(lit.bias) else( if NOT lit.raytracedShadows then lit.mapbias else lit.raytraceBias))
		mainLightSamples.value 		= if NOT lit.raytracedShadows then lit.sampleRange else if lit.raytracedShadows and hasproperty lit "resolution" then lit.filter else 4.0
		mainLightAreashadows.state 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.areaShadow else false
		mainLightTransparentshadows.state 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.transparentShadows else false
		mainLightUsize.value 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.Usize else 10.0
		mainLightVsize.value 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.Vsize else 10.0
		mainLightWsize.value 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.Wsize else 10.0
		mainLightAreashadowtype.state 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then (lit.areaShadowType+1) else 2
		mainLightSubdivs.value 		= if lit.raytracedShadows and hasProperty lit "smoothshadows" then lit.subdivs else 8
	)

-- enables/disables the elements in the light parameters box
	fn lightParamBoxEnabled enbl = (
		if NOT enbl then resetLightParam()
		grpLightParam.enabled = enbl
		mainLightSelect.enabled = enbl
		mainLightOn.enabled = enbl
		mainLightName.enabled = enbl
		mainLightMult.enabled = enbl
		mainLightColor.enabled = enbl
		mainLightShad.enabled = enbl
		mainLightShadType.enabled = enbl
		local a = getSelectedItem mainGroups.selection
		local b = getSelectedItem mainGroupLights.selection
-- create some true/false answers, and enable/disables elements concordantly... I know, it's messy, but eh, it works...
		-- property "raytracebias" is exclusive to raytraced shadows.
		-- property "smoothshadows" is exclusive to vrayshadows.
		-- property "resolution" is exclusive to vrayshadowmaps.
		try (typeRaytraced = hasproperty lightGroups[a].lights[b] "raytracebias") catch (typeRaytraced = false)
		try (typeVray = hasproperty lightGroups[a].lights[b] "smoothshadows") catch (typeVray = false)
		try (typeVrayS = hasproperty lightGroups[a].lights[b] "resolution") catch (typeVrayS = false)
		mainLightLabel_1.enabled =  (enbl and (NOT typeRaytraced) and (NOT typeVray))
		mainLightMapSize.enabled = (enbl and (NOT typeRaytraced) and (NOT typeVray))
		mainLightLabel_2.enabled = enbl
		mainLightBias.enabled = enbl
		mainLightLabel_3.enabled = (enbl and (NOT typeRaytraced) and (NOT typeVray) and (NOT typeVrayS))
		mainLightSamples.enabled = (enbl and (NOT typeRaytraced) and (NOT typeVray) and (NOT typeVrayS))
		mainLightSubdivs.enabled = (enbl and (typeVray))
		mainLightLabel_4.enabled = (enbl and (typeVray))
		mainLightLabel_5.enabled = (enbl and (typeVray))	
		mainLightLabel_6.enabled = (enbl and (typeVray))
		mainLightLabel_7.enabled = (enbl and (typeVray))
		mainLightTransparentshadows.enabled = (enbl and (typeVray))
		mainLightAreashadows.enabled = (enbl and (typeVray))
		mainLightUSize.enabled = (enbl and (typeVray))
		mainLightVSize.enabled = (enbl and (typeVray))
		mainLightWSize.enabled = (enbl and (typeVray))
		mainLightAreaShadowType.enabled = (enbl and (typeVray))
	)

-- enables/disables the elements in the lights box
	fn lightsBoxEnabled enbl = (
		grpLights.enabled = enbl
		mainGroupLights.enabled = enbl
		mainLightAll.enabled = enbl
		mainLightNone.enabled = enbl
		mainLightInv.enabled = enbl
		mainAddLights.enabled = enbl
		mainDelLights.enabled = enbl
		
		lightParamBoxEnabled (isSingleItemSelected mainGroupLights.selection)
	)

-- enables/disables the elements in the groups box
	fn groupsBoxEnabled enbl = (
		grpGroupsOutline.enabled = enbl
		grpGroups.enabled = enbl
		mainGroups.enabled = enbl
		mainGroupAll.enabled = enbl
		mainGroupNone.enabled = enbl
		mainGroupInv.enabled = enbl
		mainAddGroup.enabled = enbl
		mainDelGroup.enabled = enbl
		mainGrpName.enabled = enbl
		grpGroupState.enabled = enbl
		mainGrpEnable.enabled = enbl
		mainGrpHide.enabled = enbl
		grpGroupSelScene.enabled = enbl
		mainGrpSelClear.enabled = enbl
		mainGrpSelect.enabled = enbl
	)

-- updates the groups GUI
	fn updateGroupsGUI sel= (
		if sel == #{} then (
			mainGrpEnable.state = false
			mainGrpHide.state = false
			mainGrpEnable.text = "On"
			mainGrpHide.text = "Hide"
		) else (
			local onNum = 0
			local hideNum = 0
			local arr = (sel as array)
			for i in arr do (
				if lightGroups[i].enabled then onNum += 1
				if lightGroups[i].hide then hideNum += 1
			)
			mainGrpEnable.state = (onNum > 0)
			mainGrpHide.state = (hideNum > 0)
			mainGrpEnable.text = if (onNum == arr.count) or (onNum == 0) then "On" else "(On)"
			mainGrpHide.text = if (hideNum == arr.count) or (hideNum == 0) then "Hide" else "(Hide)"
		)
		if isSingleItemSelected sel then (
			local n = (sel as array)[1]
			mainGrpName.enabled = true
			mainGrpName.text = lightGroups[n].name
			lightsBoxEnabled true
			mainGroupLights.items = (getLightsNames lightGroups[n].lights)
		) else (
			mainGrpName.enabled = false
			mainGrpName.text = ""
			mainGroupLights.items = #()
			lightsBoxEnabled false
		)
	)

-- General Functions
------------------------------
-- updates the selected light parameters in the scene.
	
	fn updateLightParam indx = (
		local a = getSelectedItem mainGroups.selection
		local b = getSelectedItem mainGroupLights.selection
		local lit = lightGroups[a].lights[b]
		case indx of (
			1:	lit.on 						= mainLightOn.state
			2:	lit.name	 				= mainLightName.text
			3:	lit.multiplier	 			= mainLightMult.value
			4:	lit.color 					= mainLightColor.color
			5:	lit.baseobject.castShadows 	= mainLightShad.state
			6:	if mainLightShadType.selection == 3
				then lit.shadowgenerator = Vrayshadow ()
					else(
						if mainLightShadType.selection == 1
						then lit.shadowgenerator = shadowmap ()
							else(
								if mainLightShadType.selection == 4
								then  lit.shadowgenerator = vrayshadowmap ()
									else (lit.raytracedShadows 		= (mainLightShadType.selection == 2))
									) 
							)
			
			7:	if NOT lit.raytracedShadows then 
					lit.mapSize				= mainLightMapSize.value
					else(if lit.raytracedShadows and hasproperty lit "resolution" then lit.resolution			= mainLightMapSize.value)
			8:	if lit.raytracedShadows then(
				if hasproperty lit "smoothshadows" then(
					lit.Bias	 	= mainLightBias.value) 
					else(
						if  hasproperty lit "resolution" then lit.Bias	 	= mainLightBias.value else(
					lit.raytraceBias	 	= mainLightBias.value)))
				else(
				lit.mapBias 			= mainLightBias.value
					)
			-- Number 9 doesn't work - i casts crazy numbers -  but I don't care i turned that box inactive, and it does the trick
			9:	if NOT lit.raytracedShadows then 
				lit.sampleRange 		= mainLightSamples.value
				else(if lit.raytracedShadows and hasproperty lit "resolution" then lit.filter		= (mainLightMapSize.value))
						
-- SO, everything from 1 to 9 works fine !. You might have noticed, it actually is part of the old code. from 10 to 16 is what I added and ... it doesn't work at all. whatever you click, it wont update in max..
--If you dont understand the "has property" trick, please, read comments in the block dedicated to greyout out elements, above in this script.
			10: if lit.raytracedShadows then
						lit.areashadow = mainLightAreashadows.state
			11: if lit.raytracedShadows then(
				if hasproperty lit "smoothshadows" then(
					lit.baseobject.TransparentShadows = mainLightTransparentShadows.state))
			12: if lit.raytracedShadows then(
				if hasproperty lit "smoothshadows" then(
					lit.Usize	 	= mainLightUsize.value) 
					else())
			13:  if lit.raytracedShadows then( if hasproperty lit "smoothshadows" then(
					lit.Vsize	 	= mainLightVsize.value))
			14: if lit.raytracedShadows then(
				if hasproperty lit "smoothshadows" then(
					lit.Wsize	 	= mainLightWsize.value))
			15: if lit.raytracedShadows then(
				if hasproperty lit "smoothshadows" then(
					(lit.areaShadowType+1)	 	= mainLightAreashadowtype.state)		)
			16: if lit.raytracedShadows then(
				if hasproperty lit "smoothshadows" then(
					lit.subdivs	 	= mainLightSubdivs.value)) 
			
-- that's it. Further code concerns others block from the interface. 			
		)

		lightParamBoxEnabled true
	)

-- adds the lights from the select object window to the lights list, while making
-- sure there are no duplicates
	fn addLightsToList origList addList = (
		local tmpOrig = origList
		local list = #()
		try (
			list = for i in addList where (findItem origList i == 0) collect i
		) catch (return undefined)
		join tmpOrig list
		tmpOrig
	)

-- returns the first selected item (converts selection bitArray to integer to use as an index)
	fn getSelectedItem arr= (
		try (
			return (arr as array)[1]
		) catch(return undefined)
		0
	)
	
-- returns the number of selected objects in the listbox (counts the 'true's in a bitArray)
	fn bitarrayCount bitArr = (bitArr as array).count
	
-- returns true if only one object is selected in sel (sel is bitArray)
	fn isSingleItemSelected sel = (sel as array).count == 1
	
-- return an array of the names of the lights in the list (for use as items in the lights listbox)
	fn getLightsNames lightsList = (
		local g
		local names = #()
		if lightsList != undefined then
			names = for g in lightsList where (g.name != undefined) collect g.name
		names
	)

-- return an array of the names of the groups in the list (for use as items in the groups listbox)
	fn getGroupsNames groupList = (
		local g
		local names = #()
		if groupList != undefined then
			for g in groupList where (g.name != undefined) do (
				local onoff = if g.enabled then "On" else "Off"
				local hidden = if g.hide then "Hidden" else "Visible"
				append names (g.name+"   (" +onoff+","+hidden+")")
			)
		names
	)


-- Sets list manipulation functions
--------------------------------------
-- create an input box. val is the text value. title is the window title.
-- txt is the text to be desplayed in the window.
	fn inputTextBox &val title:"Enter Text" txt:"" = (
		global returnVal
		global inputTextBoxRO
		global tmpVal
		local textStr = filterString txt "\n\r"
		
		-- define the rollout
		execStr = "rollout inputTextBoxRO \"" + title + "\" (\n"
		for i = 1 to textStr.count do
			execStr += "\tlabel lbl_" + i as string + " \"" + textStr[i] + "\" align:#left\n"
		execStr += "\tedittext textVal \"\" text:\"" + val as string + "\" offset:[0,5] align:#center\n"
		execStr += "\tbutton bnOK \"OK\" width:70 across:2 offset:[0,10] align:#center\n"
		execStr += "\tbutton bnCancel \"Cancel\" width:70 offset:[0,10] align:#center\n\n"
		execStr += "\ton bnOK pressed do (\n"
		execStr += "\t\ttmpVal=textVal.text\n"
		execStr += "\t\treturnVal = true\n"
		execStr += "\t\ttry (destroyDialog inputTextBoxRO)catch()\n"
		execStr += "\t)\n"
		execStr += "\ton bnCancel pressed do (\n"
		execStr += "\t\treturnVal = false\n"
		execStr += "\t\ttry (destroyDialog inputTextBoxRO)catch()\n"
		execStr += "\t)\n"
		execStr += ")"

		execute execStr

		local windowWidth = amax (for i in textStr collect (getTextExtent i)[1] as integer)
		if (windowWidth+=30)<180 then windowWidth = 180
		try (destroyDialog inputTextBoxRO)catch()
		createDialog inputTextBoxRO width:windowWidth modal:true

		if returnVal then val = tmpVal
		returnVal
	)

-- update the groups when a new set is selected
	fn selectSet n = (
		if n == 0 then groupsBoxEnabled false
		else groupsBoxEnabled true
		try(lightGroups=groupSets[n].lightGroups)catch(lightGroups=#())
		lastGrpNum = lightGroups.count
		updateAllGroupsMode()
		mainGroups.selection = #{}
		updateGroupsGUI #{}
		gc()
	)

-- adds a new set to the setsList var
	fn addSet = (
		local setName = ""
		if (inputTextBox &setName title:"Add New Set" txt:"Enter name of new set:") then (
			if setName=="" then setName = ("Light Grouper Set"+(lastSetNum+1) as string)
			lastSetNum += 1
			local newSet = groupSet setName #()
			append groupSets newSet
			mainSetList.items = for i in groupSets collect i.name
			mainSetList.selection = mainSetList.items.count
			selectSet mainSetList.selection
			autoSave()
		)
		gc()
	)
	
-- clones the selected set and add the new set to the setsListVar.
	fn cloneSet = (
		if mainSetList.selection == 0 then return()
		local setName = ""
		if (inputTextBox &setName title:("Clone Set "+mainSetList.selected) txt:"Enter name of new set:") then (
			if setName=="" then setName = ("Light Grouper Set"+(lastSetNum+1) as string)
			lastSetNum += 1
			local tmpLightGroups = for g in groupSets[mainSetList.selection].lightGroups collect (g.copy())
			local newSet = groupSet setName tmpLightGroups
			append groupSets newSet
			mainSetList.items = for i in groupSets collect i.name
			mainSetList.selection = mainSetList.items.count
			selectSet mainSetList.selection
			autoSave()
		)
		gc()
	)

-- renames the selected set
	fn renameSet = (
		if mainSetList.selection == 0 then return()
		local setName = mainSetList.selected
		if (inputTextBox &setName title:"Rename Set" txt:("Enter new new for "+mainSetList.selected)) then (
			if setName!="" then
				groupSets[mainSetList.selection].name = setName
			mainSetList.items = for i in groupSets collect i.name
			autoSave()
		)
	)
	
-- deletes the selected set
	fn delSet = (
		if mainSetList.selection == 0 then return()
		local sel = mainSetList.selection
		if (queryBox ("Are you sure you want to delete "+mainSetList.selected) title:"Remove Set") then (
			deleteItem groupSets mainSetList.selection
			mainSetList.items = for i in groupSets collect i.name
			if mainSetList.items.count > 0 then
				if mainSetList.selection == 0 then (
					if sel > mainSetList.items.count then 
						mainSetList.selection = mainSetList.items.count
					else
						mainSetList.selection = sel
				)
			selectSet mainSetList.selection
			autoSave()
		)
	)


-- Groups list manipulation functions
---------------------------------------
-- adds a new group to the gourpsList var and update the GUI
	fn addGroup = (
		append lightGroups (lightGroup name:("Group_"+lastGrpNum as string) lights:#())
		lastGrpNum += 1
		mainGroups.items = (getGroupsNames lightGroups)
		local n = mainGroups.items.count
		mainGroups.selection = n
		updateGroupsGUI mainGroups.selection
	)
	
-- Removes the selected groups from the groupsList var and update the GUI
	fn delGroups = (
		local sel = mainGroups.selection
		if sel.count > 0 then (
			for i = sel.count to 1 by -1 where sel[i] do (deleteItem lightGroups i)
		)
		mainGroups.items = (getGroupsNames lightGroups)
		mainGroups.selection = #()
		updateGroupsGUI mainGroups.selection
	)

-- Renames the selected group	
	fn changeGroupName txt= (
		local sel = mainGroups.selection
		local n = getSelectedItem sel
		if n != undefined and bitarrayCount sel == 1 then lightGroups[n].name = txt
		mainGroups.items = (getGroupsNames lightGroups)
	)
	
-- the next two functions change the lights in the selected groups on or off
	fn changeLightsState list state = (
		try (
			for l in list do l.enabled = state
		) catch ()
	)
	
	fn changeGroupEnable state selArr: = (
		if selArr == unsupplied then sel = mainGroups.selection
		else sel = selArr
		if sel.count > 0 then (
			for i = 1 to sel.count where sel[i] do (
				lightGroups[i].enabled = state
				changeLightsState lightGroups[i].lights lightGroups[i].enabled
			)
		)
		if selArr == unsupplied then (
			mainGroups.items = (getGroupsNames lightGroups)
			if ((isSingleItemSelected mainGroupLights.selection) and \
					(isSingleItemSelected mainGroupLights.selection)) then (
				local a = getSelectedItem mainGroups.selection
				local b = getSelectedItem mainGroupLights.selection
				local lit = lightGroups[a].lights[b]
				updateLightParamGUI lit
			)
		)
	)
	
-- the next two functions hide or unhide the lights in the selected groups
	fn changeLightsHideState list state = (
		try (
			for l in list do 
				if state then 
					hide l
				else
					unhide l 
		) catch ()
	)

	fn changeGroupHide state selArr: = (
		if selArr == unsupplied then sel = mainGroups.selection
		else sel = selArr
		if sel.count > 0 then (
			for i = 1 to sel.count where sel[i] do (
				lightGroups[i].hide = state
				changeLightsHideState lightGroups[i].lights lightGroups[i].hide
			)
		)
		if selArr == unsupplied then
			mainGroups.items = (getGroupsNames lightGroups)
	)

-- update the light state and hide mode for all groups in light grouper
	fn updateAllGroupsMode = (
		disableSceneRedraw()
		for i = 1 to lightGroups.count do (
			changeGroupEnable lightGroups[i].enabled selArr:#{i}
			changeGroupHide lightGroups[i].hide selArr:#{i}
		)
		enableSceneRedraw()
		gc()
	)

-- Lights list manipulation functions
----------------------------------------
-- adds lights to the selected group
	fn addLights = (
		local grpName = mainGrpName.text
		local l = undefined
		l = selectByName title:("Select lights to add to "+grpName) buttonText:"Add" \
						filter:pickFilter showHidden:true single:false
		try (
			if l != undefined then (
				local n = getSelectedItem mainGroups.selection
				lightGroups[n].lights = (addLightsToList lightGroups[n].lights l)
				mainGroupLights.items = (getLightsNames lightGroups[n].lights)
			)
		) catch()
	)

-- removes selected lights from the list
	fn delLights = (
		local sel = mainGroupLights.selection
		local n = getSelectedItem mainGroups.selection
		try (
			if sel.count > 0 then (
				for i = sel.count to 1 by -1 where sel[i] do (deleteItem lightGroups[n].lights i)
			)
		) catch ()
		mainGroupLights.items = (getLightsNames lightGroups[n].lights)
		mainGroupLights.selection = #()
	)

-- selects the lights in the selected groups in the scene
	fn selectInScene clearSel = (
		if clearSel then clearSelection()
		local sel = mainGroups.selection
		local list = #()
		if sel.count > 0 then (
			for i = 1 to sel.count where sel[i] do (
				list = for l in lightGroups[i].lights collect l
				selectMore list
			)
		)
	)

-- Change handlers functions
--------------------------------
	fn delLightFromList obj = (
		for g in lightGroups do (
			for l in g.lights where (l == obj) do
				deleteItem g.lights (findItem g.lights obj)
		)
		if isSingleItemSelected mainGroups.selection then (
			local n = getSelectedItem mainGroups.selection
			mainGroupLights.items = (getLightsNames lightGroups[n].lights)
			mainGroupLights.selection = #()
			lightParamBoxEnabled false
		) 
	)
	
	fn renmaeLight obj = (
		if isSingleItemSelected mainGroups.selection then (
			local n = getSelectedItem mainGroups.selection
			if findItem lightGroups[n].lights obj > 0 then 
				mainGroupLights.items = (getLightsNames lightGroups[n].lights)
		) 
	)

-- Startup end cleanup functions
----------------------------------
-- Clean up the mess.
	fn done = (
		try (deleteAllChangeHandlers id:#delObj) catch(format "Could not remove #delObj change handlers.\n")
		try (deleteAllChangeHandlers id:#renameObj) catch(format "Could not remove #renameObj change handlers.\n")
		try (closeRolloutFloater lgFloat) catch(format "Could not close the floater.\n")
		autoSave()
		saveIniFile()
		gc()
		format "Thank you for using Lights Grouper.\n"
	)

	
-- Event Handlers
-------------------
	on mainSetList selected item do (selectSet item)
	on mainAddSet pressed do (addSet())
	on mainCopySet pressed do (cloneSet())
	on mainRenameSet pressed do (renameSet())
	on mainDelSet pressed do (delSet())

	on mainGroups selectionEnd do (
		mainGroupLights.selection = #()
		updateGroupsGUI mainGroups.selection
	)
	on mainGroupAll pressed do (
		mainGroups.selection = #{1..mainGroups.selection.count}
		updateGroupsGUI mainGroups.selection
	)
	on mainGroupNone pressed do (
		mainGroups.selection = #()
		updateGroupsGUI mainGroups.selection
	)
	on mainGroupInv pressed do (
		mainGroups.selection = selectionInvert mainGroups.selection
		updateGroupsGUI mainGroups.selection
	)
	
	on mainAddGroup pressed do (addGroup(); autoSave())
	on mainDelGroup pressed do (delGroups(); autoSave())

	on mainGrpName changed text do (changeGroupName text)
	on mainGrpName entered text do (autoSave())
	on mainGrpEnable changed state do (changeGroupEnable state; autoSave())
	on mainGrpHide changed state do (changeGroupHide state; autoSave())

	on mainGrpSelect pressed do (selectInScene mainGrpSelClear.state)
	
	on mainAddLights pressed do (addLights(); autoSave(); createChangeHandlers())
	on mainDelLights pressed do (delLights(); autoSave(); createChangeHandlers())

	on mainLightAll pressed do (
		mainGroupLights.selection = #{1..mainGroupLights.selection.count}
		lightParamBoxEnabled false
	)
	on mainLightNone pressed do (
		mainGroupLights.selection = #()
		lightParamBoxEnabled false
	)	
	on mainLightInv pressed do (
		mainGroupLights.selection = selectionInvert mainGroupLights.selection
		if (isSingleItemSelected mainGroupLights.selection) then (
			lightParamBoxEnabled true
			local a = getSelectedItem mainGroups.selection
			local b = getSelectedItem mainGroupLights.selection
			local lit = lightGroups[a].lights[b]
			updateLightParamGUI lit
		) else (
			lightParamBoxEnabled false
		)
	)

	on mainGroupLights selectionEnd do (
		if (isSingleItemSelected mainGroupLights.selection) then (
			lightParamBoxEnabled true
			local a = getSelectedItem mainGroups.selection
			local b = getSelectedItem mainGroupLights.selection
			local lit = lightGroups[a].lights[b]
			updateLightParamGUI lit
		) else (
			lightParamBoxEnabled false
		)
	)
	on mainLightSelect pressed do (
		local a = getSelectedItem mainGroups.selection
		local b = getSelectedItem mainGroupLights.selection
		select (lightGroups[a].lights[b])
	)

	on mainLightOn changed state do (updateLightParam 1)
	on mainLightName entered text do (
		updateLightParam 2
		local n = getSelectedItem mainGroups.selection
		mainGroupLights.items = (getLightsNames lightGroups[n].lights)
		autoSave()
	)
	on mainLightMult changed value do (updateLightParam 3)
	on mainLightColor changed color do (updateLightParam 4)
	on mainLightShad changed state do (updateLightParam 5)
	on mainLightShadType selected selection do (updateLightParam 6)
	on mainLightMapSize changed value do (updateLightParam 7)
	on mainLightBias changed value do (updateLightParam 8)
	on mainLightSamples changed value do (updateLightParam 9)
	
	on closeFloater pressed do (
		try (removeRollout Main) catch()
		try (removeRollout saveLoad) catch()
		try (removeRollout aboutRO) catch()
	)

	on main open do (loadFromMaxScene())
	on Main close do (format "Closing...\n"; done())

) -- End Main rollout


--***********************************************************************
-- saveLoad rollout
--***********************************************************************
rollout saveLoad "Save/Load 'Light Grouper' data" rolledUp:true
(
-- Local Function Declarations
-------------------------------

	
-- GUI
-------------
	group "MAX scene " (
		button slSaveScene "Save" width:70 pos:[10,25] toolTip:"Save date to scene"
		button slLoadScene "Load" width:70 pos:[90,25] toolTip:"Load data from scene"
		button slDeleteScene "Delete From Scene" pos:[170,25] toolTip:"Remove Light Grouper data from max scene"
		checkbutton slAutoSave "Auto Save" width:70 checked:true pos:[300,25] toolTip:"Automatically saves every change in Lights Grouper to the max scene"
	)
	group "External file " (
		button slSaveFile "Save" width:70 pos:[10,75] toolTip:"Load data to file"
		button slLoadFile "Load" width:70 pos:[90,75] toolTip:"Load data from file"
	)

	
-- Functions
--=====================================================================
-- GUI functions
------------------
-- This function updates the Light Groups rollout GUI
	fn updateGUI = (
		main.mainSetList.items = for i in main.groupSets collect i.name
		try (main.mainSetList.selection = 1) catch()
		main.selectSet main.mainSetList.selection
		main.mainGroups.items = (main.getGroupsNames main.lightGroups)
		main.mainGroupLights.items = #()
		main.updateGroupsGUI main.mainGroups.selection
	)

-- General functions
-----------------------
-- strips the <light></light> tags of the string and returns the light name	
	fn getLightName str = (
		local start = findString str "<light>"
		local end = findString str "</light>"
		if  start != undefined and end != undefined then
			(subString str (start+7) (end-start-7))
		else undefined
	)
	

-- this function clears the light grouper data stored in the appData 
	fn clearFromMaxScene = (
		local baseAddr = appDataAddr
		local addr = 1
		try (
			while (getAppData globalTracks baseAddr != undefined) do (
				deleteAppData globalTracks baseAddr
				while (getAppData globalTracks (baseAddr+addr) != undefined) do (
					deleteAppData globalTracks (baseAddr+addr)
					addr += 1
				)
				baseAddr += 100
				addr = 1
			)
		) catch (messagebox "Error trying to clear data from max scene." title:"Lights Grouper")
	)

-- this function does the actual writing to the file. it is used by the saveToExtFile function
	fn writeDataToExtFile file grpSet = (
		try (
			format "<set name:%>\n" grpSet.name to:file
			for grp in grpSet.lightGroups do (
				format "\t<group name:% enable:% hide:%>\n" grp.name grp.enabled grp.hide to:file
				for l in (main.getLightsNames grp.lights) do 
					format "\t\t<light>%</light>\n" (l as string) to:file
				format "\t</group>\n" to:file
			)
			format "</set>\n" to:file
		) catch (messagebox "Error write data to ext file")
	)
	
-- this function does the actual reading from the file. it is used by the loadFromExtFile function
	fn readFromExtFile file = (
		local progCount = 0
		try (l = readLine file)catch()
		if (findString l "<LightGrouperFile") == undefined or (findString l "ver:1.2") == undefined then
			return()
		while (NOT eof file) do (
			l = readLine file

			if findString l "<set " != undefined then (
				local namePos = (findString l "name:")
				local nameEnd = findString l ">"
				if nameEnd == undefined then nameEnd = l.count
				local nameStr = subString l (namePos+5) (nameEnd-namePos-5)
				append main.groupSets (main.groupSet nameStr #())
				main.lastSetNum += 1
			)

			if findString l "</set>" != undefined then (
				main.groupSets[main.groupSets.count].lightGroups = main.lightGroups
				main.lightGroups = #()
			)

			if findString l "<group " != undefined then (
				local namePos = (findString l "name:")
				local enblPos = (findString l "enable:")
				local hidePos = (findString l "hide:")
				local nameStr = subString l (namePos+5) (enblPos-namePos-6)
				local enblStr = subString l (enblPos+7) (hidePos-enblPos-8)
				local hideStr = subString l (hidePos+5) 5
				if nameStr == undefined or nameStr=="" then nameStr = ("Group_"+main.lastGrpNum as string)
				append main.lightGroups (main.lightGroup name:nameStr enabled:(enblStr=="true") hide:(hideStr=="true") lights:#())
				main.lastGrpNum += 1
			) 
		
			if ((findString l "<light>") != undefined) then (
				local o = getNodeByName (getLightName l)
				if o != undefined then (
					append main.lightGroups[main.lightGroups.count].lights o
				) else (
					format "% couldn't be found in the scene, and will be removed from %\n" (getLightName l) nameStr
					errNumMissingLights += 1
				)
			)
			-- display some progress in the progressbar
			try(
				progCount += 1
				lgProgress.progBar.value = (mod progCount 100)as integer
			)catch()
		)
		return OK
	)

-- this function does the actual reading from the file. it is used by the loadFromExtFile function
	fn readFromExtFileVer1_0 file = (
		local progCount = 0
		try (
			while (NOT eof file) do (
				l = readLine file
	
				if findString l "<group " == 1 then (
					local namePos = (findString l "name:")
					local enblPos = (findString l "enable:")
					local hidePos = (findString l "hide:")
					local nameStr = subString l (namePos+5) (enblPos-namePos-6)
					local enblStr = subString l (enblPos+7) (hidePos-enblPos-8)
					local hideStr = subString l (hidePos+5) 5
					if nameStr == undefined or nameStr=="" then nameStr = ("Group_"+main.lastGrpNum as string)
					append main.lightGroups (main.lightGroup name:nameStr enabled:(enblStr=="true") hide:(hideStr=="true") lights:#())
					main.lastGrpNum += 1
				) 
			
				if ((findString l "<light>") != undefined) then (
					local o = getNodeByName (getLightName l)
					if o != undefined then (
						append main.lightGroups[main.lightGroups.count].lights o
					) else (
						format "% couldn't be found in the scene, and will be removed from %\n" (getLightName l) nameStr
						errNumMissingLights += 1
					)
				)
				-- display some progress in the progressbar
				try(
					progCount += 1
					lgProgress.progBar.value = (mod progCount 100)as integer
				)catch()
			)
			append main.groupSets (main.groupSet "Light Grouper Set1" main.lightGroups)
		) catch (format "Error reading file.\n")
	)

-- Event Handlers
--=======================================================================
	on slSaveScene pressed do (saveToMaxScene())
	on slLoadScene pressed do (loadFromMaxScene())
	on slDeleteScene pressed do (clearFromMaxScene())
	
	on slSaveFile pressed do (saveToExtFile())
	on slLoadFile pressed do (loadFromExtFile())


) -- End saveLoad rollout


--***********************************************************************
-- About rollout
--***********************************************************************
rollout AboutRO "About" rolledUp:true
(

-- Local Variable Declerations
-------------------------------
	-- the text to display, each array item is a line to be scrolled.
	local txt = (" 
Description
----------------------
This utility allows you to define groups of lights, for easy management.
You can then turn all the lights in the group On/Off and Hide/Unhide
them. It is useful for task such as having the same scene with day/night
light schemes.
The new Sets feature allows to create presets of groups and easily switch
between them. This is useful, for example, if you have a large amount of
lights in the scene, and you want to create a simple lighting sceme for draft
renderings. Just create two sets, one for draft and one for production, define
which lights you want on and which off in each set, and just switch to the 
right set before you render.

*******************************************************************************************

You may use this script freely as you see fit.
You may use parts or the script as a whole in your own scripts.
(it would be nice if you give me a credit if you do so ;))
If you find this script useful, or use it production,
I would really like to know about it.
 
This script comes with no waranty!
Although I tried this script and couldn't find any problems with it, I can
in no way be held responsible for any kind of loss or damage, whether
direct or indirect, due to the use of this script.
 
******************************************************************************
IF YOU DON'T LIKE THE ABOVE STATEMENT, 
PLEASE DON'T USE THIS SCRIPT !
******************************************************************************
 
This script was written for max 4.2 and wasn't tested on other 
versions of max.
 
If you find any bugs in this script, please let me know.
") -- end of scroll text definition


	-- determines the text alignment (options:#left, #right, #center)
	local textAlign = #center	
	
	-- determines the scroll speed delay (in msecs)
	local scrollSpeed = 90
	
	------------------------------------------------------------------
	-- Init some inner vars
	-- Do NOT change these vars !!!
	local lineNum = 1
	local lineHeight = 17
	local lineOffset = 0
	local controlsList = #()


-- GUI
--------
	label aboutLbl_01 "Lights Grouper v1.2 for Max 4.2"
	label aboutLbl_02 "by " across:3 align:#center offset:[52,0]
	hyperLink aboutMail "Ofer Zelichover" address:"mailto:ofer_z@hotmail.com" color:blue hoverColor:red align:#center offset:[-23,0]
	label aboutLbl_03 " (c) 12/2001" align:#center offset:[-73,0]
	hyperLink aboutSite "(www.tdp.nu/ofer)" address:"http://www.tdp.nu/ofer" color:blue hoverColor:red align:#center offset:[0,-3]
	group "" (
		label blankRow align:textAlign offset:[0,-5]
		label scrllbl01 align:textAlign
		label scrllbl02 align:textAlign
		label scrllbl03 align:textAlign
		label scrllbl04 align:textAlign
		label scrllbl05 align:textAlign
		label scrllbl06 align:textAlign
		label scrllbl07 align:textAlign
		label scrllbl08 align:textAlign
		label scrllbl09 align:textAlign
	)
	
	timer scrollTimer interval:scrollSpeed active:false
	

-- Functions
--------------
	fn textToArray &t = (
		t = filterString t "\n\r"
	)

	fn emptyList n = (
		local tmpList = #()
		for i = 1 to n do
			append tmpList ""
		return tmpList
	)
	
	fn getScrollLabels = (
		local l = #()
		l = for i in AboutRO.controls where ((findString (i as string) "scrllbl")!=undefined) collect i
		return l
	)
	
	fn pushText txt = (
		for i = 1 to (controlsList.count-1) do
			controlsList[i].text = controlsList[i+1].text
		controlsList[controlsList.count].text=txt
	)
	
	fn offsetLines = (
		for c in controlsList do
			c.pos.y -= 1
	)
	
	fn resetLinesOffset = (
		for c in controlsList do
			c.pos.y += lineHeight+1
	)
	
	fn scrollText clearList:true =
	(
		if  lineOffset >= lineHeight then (
			lineOffset = 0
			resetLinesOffset()
			if lineNum>txt.count then (tmptxt="") else (tmptxt=txt[lineNum])
			pushText tmptxt
			if clearList then (
				if (lineNum += 1)>(txt.count+controlsList.count) then lineNum = 1
			) else (
				if (lineNum += 1)>txt.count then lineNum = 1
			)
		) else (
			lineOffset += 1
		)
		offsetLines()
	)


-- Event Handlers
-------------------
	on scrollTimer tick do (if AboutRO.open then scrollText())

	on AboutRO open do (
		textToArray &txt
		controlsList = getScrollLabels()
		scrollTimer.active = true
	)
	on AboutRO close do (
		scrollTimer.active = false
	)
	
) -- End about rollout


--***********************************************************************
-- General macroScript functions
--***********************************************************************
-- converts string point2 into point2
	fn str2point2 s = (
		local p = filterString s "[,]"
		[p[1] as integer,p[2] as integer]
	)


-- save/load functions
--------------------------
-- saves the data to AppData so it will be saved with the max scene
-- the data will be saved from AppData 33100 and on dinamiclly on
-- globalTracks. each group data is stored in a single appData index. 
	fn saveToMaxScene = (
		saveLoad.clearFromMaxScene()
		local baseAddr = appDataAddr
		local addr = 1
		for s in main.groupSets do (
			try(
				local tmpLightGroups = s.lightGroups
				-- save set name
				local list = stringStream ""
				print ("set:") to:list
				print (s.name) to:list
				setAppData globalTracks baseAddr (list)

				-- save groups
				for g in tmpLightGroups do (
					local list = stringStream ""
					print ("group:") to:list
					print (g.name) to:list
					print (g.enabled as string) to:list
					print (g.hide as string) to:list
					print "Lights:" to:list
					for l in g.lights do (print l.name to:list)
					setAppData globalTracks (baseAddr+addr) (list)
					addr += 1
				)
				baseAddr += 100
				addr = 1
			) catch (messagebox "Error write data to max scene")
		)
		gc()
	)
	
-- loads lights grouper data (ver 1.2 and on) from the max scene appData.
	fn loadFromMaxSceneNew = (
		errNumMissingLights = 0
		local baseAddr = appDataAddr
		local addr = 1
		local progCount = 0
		try (
			while (getAppData globalTracks baseAddr != undefined) do (
				local tmpSetName = ""
				main.lightGroups = #()
				main.lastGrpNum = 0

				local ss = stringStream (getAppData globalTracks baseAddr)
				if NOT eof ss then
					local str = readValue ss
				if (findString str "set:" == 1) then 
					try(tmpSetName=readValue ss)catch(tmpSetName="Light Grouper Set "+(main.lastSetNum+1) as string)

				while (getAppData globalTracks (baseAddr+addr) != undefined) do (
					local ss = stringStream (getAppData globalTracks (baseAddr+addr))
					if not eof ss then
						local str = readValue ss
					if (findString str "group:" == 1) then (
						try (
							local nameStr = readValue ss  
							local enblStr = (readValue ss) == "true"
							local hideStr = (readValue ss) == "true"
						) catch (nameStr = ("Group_"+(main.lastGrpNum+1) as string))
						append main.lightGroups (main.lightGroup name:nameStr enabled:enblStr hide:hideStr lights:#())
						main.lastGrpNum += 1
					)
					if not eof ss then
						str = readValue ss
					if (findString str "lights:" == 1) then (
						while NOT eof ss do (				
							local lightStr = readValue ss  
							local l = (getNodeByName lightStr)
							if l != undefined then (
								append main.lightGroups[main.lightGroups.count].lights l
							) else (
								format "% couldn't be found in the scene, and will be removed from % in set %\n" lightStr nameStr tmpSetName
								errNumMissingLights += 1
							)
							-- display some progress in the progressbar
							try(
								progCount += 1
								lgProgress.progBar.value = (mod progCount 100)as integer
							)catch()
						)
					)
					addr += 1
				)
				local newSet = main.groupSet tmpSetName main.lightGroups
				append main.groupSets newSet
				main.lastSetNum += 1
				baseAddr += 100
				addr = 1
			)
		) catch (messagebox "Error reading from max scene" title:"Lights Grouper")

		if errNumMissingLights > 0 then
			messagebox ("There were " + errNumMissingLights as string + " lights saved in the Lights Grouper data that were missing from the scene.\n Plsease see listener for a detailed list.") title:"Lights Grouper"

		gc()
	)

-- loads lights grouper data (ver 1.0) from the max scene appData
	fn loadFromMaxSceneVer1_0 = (
		local progCount = 0
		errNumMissingLights = 0
		local addr = appDataAddr
		try (
			while (getAppData globalTracks addr != undefined) do (
				local ss = stringStream (getAppData globalTracks addr)
				if not eof ss then
					local str = readValue ss
				if (findString str "group:" == 1) then (
					try (
						local nameStr = readValue ss  
						local enblStr = (readValue ss) == "true"
						local hideStr = (readValue ss) == "true"
					) catch (nameStr = ("Group_"+(main.lastGrpNum+1) as string))
					append main.lightGroups (main.lightGroup name:nameStr enabled:enblStr hide:hideStr lights:#())
					main.lastGrpNum += 1
				)
				if not eof ss then
					str = readValue ss
				if (findString str "lights:" == 1) then (
					while NOT eof ss do (				
						local lightStr = readValue ss  
						local l = (getNodeByName lightStr)
						if l != undefined then (
							append main.lightGroups[main.lightGroups.count].lights l
						) else (
							format "% couldn't be found in the scene, and will be removed from %\n" lightStr nameStr
							errNumMissingLights += 1
						)
						-- display some progress in the progressbar
						try(
							progCount += 1
							lgProgress.progBar.value = (mod progCount 100)as integer
						)catch()
					)
				)
				addr += 1
			)
		) catch (messagebox "Error reading from max scene" title:"Lights Grouper")
		local newSet = main.groupSet "Light Grouper Set1" main.lightGroups
		append main.groupSets newSet
		main.lastSetNum = 1

		if errNumMissingLights > 0 then
			messagebox ("There were " + errNumMissingLights as string + " lights saved in the Lights Grouper data that were missing from the scene.\n Plsease see listener for a detailed list.") title:"Lights Grouper"

	)

-- checks what format the data is saved in (ver 1.0 or 1.2) and loads using the 
-- appropriate fn.
	fn loadFromMaxScene = (
		main.groupSets = #()
		main.lastSetNum = 0
		main.lightGroups = #()
		main.lastGrpNum = 0
		local ss = (getAppData globalTracks appDataAddr)
		if ss != undefined then (
			createDialog lgProgress width:300
			try (
				ss = ss as stringStream
				local str = readValue ss
				case str of (
					"set:"		: loadFromMaxSceneNew()
					"group:"	: loadFromMaxSceneVer1_0()
				)
			) catch()
			try(destroyDialog lgProgress)catch()
		)
		saveLoad.updateGUI()
		createChangeHandlers()
	)

-- saves Lights Grouper data to a file
	fn saveToExtFile = (
		local fileName = getSaveFileName types:"Lights Grouper Files (*.dat)|*.dat|All Files (*.*)|*.*"
		if fileName != undefined then (
			f = createFile fileName
			format "<LightGrouperFile ver:%>\n" (1.2 as string) to:f
			for s in main.groupSets do (
				saveLoad.writeDataToExtFile f s
			)
			flush f
			close f
			messagebox "Done saving to file"
		)
		gc()
	)
	
-- loads Lights Grouper data from a file
	fn loadFromExtFile = (
		errNumMissingLights = 0
		local fileName = getOpenFileName types:"Lights Grouper Files (*.dat)|*.dat|All Files (*.*)|*.*"
		if fileName != undefined then (
			f = openFile fileName 
			if f != undefined then (
				createDialog lgProgress width:300
				main.groupSets = #()
				main.lastSetNum = 0
				main.lightGroups = #()
				main.lastGrpNum = 0
				if (saveLoad.readFromExtFile f) == undefined then (
					seek f 0
					saveLoad.readFromExtFileVer1_0 f
				)
				try(destroyDialog lgProgress)catch()
			)
			flush f
			close f
			saveLoad.updateGUI()
			if saveLoad.slAutoSave.state then
				saveToMaxScene()
			if errNumMissingLights > 0 then
				messagebox ("There were " + errNumMissingLights as string + " lights in the file that were missing from the scene.\n Plsease see listener for a detailed list.") title:"Lights Grouper"
			createChangeHandlers()
		)
		gc()
	)

-- save prefrences to ini file
-------------------------------
	fn saveIniFile = (
		local fName = getDir #scripts + "\\LightGrouper.ini"
		setINISetting fName "General Settings" "Pos" (lgFloat.pos as string)
		setINISetting fName "General Settings" "Size" (lgFloat.size as string)
		setINISetting fName "General Settings" "AutoSaveOn" (saveLoad.slAutoSave.checked as string)
	)

-- load prefrences from ini file
-------------------------------
	fn loadIniFile = (
		local fName = getDir #scripts + "\\LightGrouper.ini"
		try (
			lgFloat.pos = str2point2 (getINISetting fName "General Settings" "Pos")
			lgFloat.size = str2point2 (getINISetting fName "General Settings" "Size")
			saveLoad.slAutoSave.checked = (getINISetting fName "General Settings" "AutoSaveOn")=="true"
		) catch()
	)

-- Create the Change Handlers
------------------------------
	fn createChangeHandlers = (
		try (deleteAllChangeHandlers id:#delObj) catch()
		try (deleteAllChangeHandlers id:#renameObj) catch()
		
		local list = #()
		for g in main.lightGroups do (
			tmplist = for l in g.lights where (l != undefined) collect l
			join list tmplist
		)
		if list != #() then (
			when list deleted id:#delObj obj do (main.delLightFromList obj)
			when names list change id:#renameObj obj do (main.renmaeLight obj)
		)
	)


-- Create the floater window
-----------------------------
try (closeRolloutFloater lgFloat) catch()
lgFloat = newRolloutFloater "Light Grouper Utility v1.2" 410 480 10 100


-- Add the rollouts
---------------------
addRollout Main lgFloat
addRollout saveLoad lgFloat
addRollout aboutRO lgFloat

-- load the save prefs.
loadIniFile()

) -- End of macroscript
